// This software including the source code and documentation (the "SOFTWARE")
// is copyrighted.  You are free to use the SOFTWARE for any non-commercial
// purpose whatsoever provided you do not remove this copyright notice.
// If you wish to use the SOFTWARE for commercial purposes, please contact us
// to obtain a license.  The SOFTWARE is provided "as is", with absolutely no
// warranty.  Use it at your own risk.
// Website: http://www.math.ie/checker.html

#ifndef _bitmask_H
#define _bitmask_H   1

class BitMask
   {
   public:
      BitMask() : m_cBits( 0 ), m_cInts( 0 ), m_pInts( NULL ) {}
      BitMask( BitMask const& rhs )
         {
         m_pInts = NULL;
         *this = rhs;
         }

      void operator=( BitMask const& rhs )
         {
         m_cBits = rhs.m_cBits;
         m_cInts = rhs.m_cInts;
         delete m_pInts;
         m_pInts = new int[ m_cInts ];
         memcpy( m_pInts, rhs.m_pInts, m_cInts * sizeof( int ) );
         }

      ~BitMask() { delete m_pInts; }

      void SetDimension( int cBits )
         {
         delete m_pInts;
         m_cBits = cBits;
         m_cInts = (cBits + 31) / 32;
         m_pInts = new int[ m_cInts ];
         memset( m_pInts, 0, m_cInts * sizeof( int ) );
         }

      void operator|=( BitMask const& rhs )
         {
         for ( int i = 0;  i < m_cInts;  i++ )
            m_pInts[ i ] |= rhs.m_pInts[ i ];
         }

      void SetBit( int iBit )
         {
         m_pInts[ iBit / 32 ] |= 1 << (iBit % 32);
         }

      // false = all bits set
      bool GetIndexOfLowestBitNotSet( int& iBit ) const
         {
         iBit = 0;
         for ( int i = 0;  i < m_cInts;  i++ )
            {
            if ( m_pInts[ i ] == 0xFFFFFFFF )
               iBit += 32;
            else
               {
               int iOffset = 0;
               if ( (m_pInts[ i ] & 0xFFFF) == 0xFFFF )
                  iOffset += 16;
               if ( (m_pInts[ i ] & (0xFF << iOffset)) == (0xFF << iOffset) )
                  iOffset += 8;
               if ( (m_pInts[ i ] & (0xF << iOffset))  == (0xF << iOffset) )
                  iOffset += 4;
               if ( (m_pInts[ i ] & (0x3 << iOffset))  == (0x3 << iOffset) )
                  iOffset += 2;
               if ( (m_pInts[ i ] & (0x1 << iOffset))  == (0x1 << iOffset) )
                  iOffset += 1;
               iBit += iOffset;
               return (iBit < m_cBits);
               }
            }

         return false;
         }

   private:
      int   m_cBits;
      int   m_cInts;
      int*  m_pInts;
   };

#endif
