// This software including the source code and documentation (the "SOFTWARE")
// is copyrighted.  You are free to use the SOFTWARE for any non-commercial
// purpose whatsoever provided you do not remove this copyright notice.
// If you wish to use the SOFTWARE for commercial purposes, please contact us
// to obtain a license.  The SOFTWARE is provided "as is", with absolutely no
// warranty.  Use it at your own risk.
// Website: http://www.math.ie/checker.html

#ifndef _grid_H
#define _grid_H 1

#include <memory.h>
#include <stdio.h>
#include <stdlib.h>
#include "myset.h"
#include <string>

// the maximum size of an unavoidable set
int const MAX_UNAV_SET = 24;

// models an unavoidable set
class UnavSet
   {
   public:
      int size() const { return m_cbData; }
      int operator[]( int i ) const { return m_aiData[ i ]; }
      UnavSet const& operator=( UnavSet const& rhs ) { rhs; printf( "Not allowed." ); exit( 0 ); }
      void SetData( int const* piData, int cbData )
         {
         memcpy( m_aiData, piData, cbData * sizeof( int ) );
         m_cbData = cbData;
         ComputeDeadClueMasks();
         }

      void SetData( MySet const* pMySet )
         {
         m_cbData = pMySet->_count;
         memcpy( m_aiData, pMySet->_set, sizeof( int ) * m_cbData );
         ComputeDeadClueMasks();
         }

      void Print() const
         {
         cout << "   {";
         for ( int i = 0;  i < m_cbData;  i++ )
            {
            cout << m_aiData[ i ];
            if ( i != m_cbData - 1 )
               cout << ",";
            }
         cout << "}" << endl;
         }

   private:
      void ComputeDeadClueMasks()
         {
         // initialise clue mask for first clue in this unavoidable set
         m_aiDeadClues[ 0 ][ 0 ] = 0;
         m_aiDeadClues[ 0 ][ 1 ] = 0;
         m_aiDeadClues[ 0 ][ 2 ] = 0;
         int idx = Coords2Idx( m_aiData[ 0 ] );
         m_aiDeadClues[ 0 ][ idx / 32 ] = (1 << (idx % 32));
         // now set up masks for subsequent clues in this unavoidable set
         for ( int i = 1;  i < size();  i++ )
            {
            // first copy mask from previous clue
            m_aiDeadClues[ i ][ 0 ] = m_aiDeadClues[ i - 1 ][ 0 ];
            m_aiDeadClues[ i ][ 1 ] = m_aiDeadClues[ i - 1 ][ 1 ];
            m_aiDeadClues[ i ][ 2 ] = m_aiDeadClues[ i - 1 ][ 2 ];
            // finally mark the current clue as dead also
            idx = Coords2Idx( m_aiData[ i ] );
            m_aiDeadClues[ i ][ idx / 32 ] |= (1 << (idx % 32));
            }
         }

   public:
      int m_aiDeadClues[ MAX_UNAV_SET ][ 3 ];

   private:
      int m_aiData[ MAX_UNAV_SET ];
      int m_cbData;
   };



// a (completed) 9x9 Sudoku grid
// constructor displays and error message and immediately terminates program if
// grid could not be read from file successfully
class Grid
   {
   public:
      Grid( char const* pszFilename, char const* pszFilenameExtraUnavoidables );
      ~Grid();
      char const* GetGrid() const { return m_pszGrid; }
      int GetNumberOfDisj() const { return m_cDisjoints; }
      int GetNumberOfUnav() const { return m_cUnav; }
      UnavSet const* GetUnav() const { return m_pUnav; }

   private:
      // computes the number of unavoidable sets, the first <return_value>
      // sets are a maximal collection of disjoint unavoidable sets
      void ComputeUnavoidableSets( char const* pszGrid,
                                   char const* pszFilenameExtraUnavoidables,
                                   UnavSet** ppUnav, int* pcUnav, int* pcDisj );
   private:
      char*       m_pszGrid;
      UnavSet*    m_pUnav;
      int         m_cUnav;
      int         m_cDisjoints;
      std::string m_sFileUnav;
   };

#endif
